/*================================================================================
*
*   Allen Mulvey June 2014
*   DS18B20_Get_ID
*        This retrieves the unique ID from the DS18B20
*        and stores it in the PIC EEPROM for future reference.
*
*        You can use the features of the mikroProg Suite to reprogram 
*        the PIC without clearing the data EEPROM. This way you can
*        transfer several IDs to the EEPROM then reprogram the PIC to use them.
*
*        The ID is also displayed on the LCD in case you want to do this manually.
*
*        1) In the MikroC editor BUILD the project to create hex and ihex files.
*        2) Open the mikroProg Suite.
*        3) check "Load/Save CODE"
*        4) uncheck "Load/Save DATA"
*        5) click "Options" and check "Preserve DATA"
*        6) click "Load" Hex File and select your file
*        7) click "Write"
*        This will reprogram the PIC without erasing the EEPROM.
*        Do this as many times as necessary to save all the IDs then
*        do it one last time to put your own code in.
*
*        You can also save the EEPROM as an ihex file to include in another project.
*
*        Be sure to save each ID to a different location in the EEPROM.
*        Use the constant EEPROM_offset to set the start location for the ID.
*        Each ID is 8 bytes long.
*
================================================================================*/

const char EEPROM_offset = 0x30;             // Set your offset here ****************************************
char SlaveROM[8];
float temperature;

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;


// I use a procedure for any delay that is used more than once
void Comm_Delay() {Delay_us(10);}       // You can experiment with this delay
void Delay_10()   {Delay_ms(10);}
void Delay_20()   {Delay_ms(20);}
void Delay_200()  {Delay_ms(200);}
void Delay_100()  {Delay_ms(100);}
void Delay_500()  {Delay_ms(500);}

void getSlaveROM(){
    char i, tmp[3];
    INTCON.GIE = 0;
    Ow_Reset(&PORTE, 2);                         // Onewire reset signal
    Delay_us(140);
    Ow_Write(&PORTE, 2, 0x33);                   // Issue command READ_ROM
    Delay_us(140);
    for(i=0; i<8; i++)SlaveROM[i] =  Ow_Read(&PORTE, 2);   // LSB first
    INTCON.GIE = 1;
    for(i=0; i<8; i++)EEPROM_write(i+EEPROM_offset, SlaveROM[i]);   // LSB first
}

void getDS18B20(){
    unsigned int temp;
    INTCON.GIE = 0;
    Ow_Reset(&PORTE, 2);                      // Onewire reset signal
    Delay_us(140);
    Ow_Write(&PORTE, 2, 0xCC);                // Issue command SKIP_ROM
    Ow_Write(&PORTE, 2, 0x44);                // Issue command CONVERT_T

    Ow_Reset(&PORTE, 2);
    Delay_us(140);

    Ow_Write(&PORTE, 2, 0xCC);                // Issue command SKIP_ROM
    Ow_Write(&PORTE, 2, 0xBE);                // Issue command READ_SCRATCHPAD
    temp =  Ow_Read(&PORTE, 2);
    temp = (Ow_Read(&PORTE, 2) << 8) + temp;
    temperature = (temp & 0x000F) * (.0625);  // get decimal part
    temperature = (temperature + (temp >> 4)) * 9 / 5 + 32;

    INTCON.GIE = 1;
}

void Display_It(){
     char txt[7], tmp[3];
     int i;
     Lcd_Cmd(_LCD_CLEAR);
     byteToStr(EEPROM_offset, tmp);
     Lcd_Out(1, 14, tmp);
     i = temperature;      // convert to INT
     IntToStr(i, txt);
     Lcd_Out(1, 4, txt);
     Lcd_Out(1, 1, "Temp:");
     for(i=0; i<8; i++){ByteToHex(SlaveROM[i],tmp); Lcd_Out(2, 15-2*i, tmp);}
     Delay_500();
}

// =============================================================================
// =============================================================================

void main(){
       INTCON = 0;       // turn off all interrupts
       ADCON0 = 0;       // Turn off analog to digital converters
       ANSELA = 0;       // Configure all ports as digital
       ANSELB = 0;
       ANSELC = 0;
       ANSELD = 0;
       ANSELE = 0;

       /*LATA     = 0;
       TRISA    = 0;     // outputs
       LATB     = 0;
       TRISB    = 0;
       LATC     = 0;
       TRISC    = 0;*/
       LATD     = 0;
       TRISD    = 0xff;
       LATE     = 0;
       TRISE    = 4;

       getDS18B20(); while(temperature == 185) getDS18B20();
       Lcd_Init();                              // Initialize LCD
       Delay_10();
       Lcd_Cmd(_LCD_CLEAR);
       Delay_10();
       Lcd_Cmd(_LCD_CURSOR_OFF);
       Delay_10();

       getSlaveROM();
       do {
           getDS18B20();
           Display_It();
       }while(1);
}